# ____________________________________________________________________
#                   BDServer Modern Search Plugin
#                      
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# ____________________________________________________________________

PluginData = {
    "author":{
        "title":"Madiator2011",
        "url":"https://shark.madiator.com/@madiator2011",
        "avatar":"https://shark.madiator.com/proxy/avatar.webp?url=https%3A%2F%2Fshark.madiator.com%2Ffiles%2F8076a50d-49db-4632-9d21-2e93ed53c500&avatar=1"
    },
    "title":"Modern Search",
    "description":"Make your search look less ugly.",
    "license":"AGPL-3.0-or-later"}

import re
import html
import os
import sys
import json
import urllib.parse

# Add the BDServer modules path
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
try:
    from modules import Set
    from modules import Render
except:
    # Fallback if imports fail
    pass

def onHTML(server, html_content):
    """Modern search plugin - applies modern styling to search pages."""
    
    # Check if we're on a search page
    if not server.path.startswith("/search"):
        return html_content
    
    # For search pages, completely replace the HTML
    html = generate_modern_search_page(server)
    
    return html

def generate_modern_search_page(server):
    """Generate a complete modern search page."""
    
    # Parse search parameters
    search_params = parse_search_params(server)
    
    # Load data
    try:
        config = Set.Load()
        user = Render.validate(getattr(server, 'cookie', None))
        
        # Perform search
        search_results = perform_search(search_params)
        
    except Exception as e:
        print(f"Error loading search data: {e}")
        search_results = []
        config = {}
        user = None
    
    # Generate HTML
    html = f"""<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Search{' - ' + search_params['text'] if search_params['text'] else ''} - {safe_html(config.get('title', 'My Website'))}</title>
    <meta name="description" content="Search results">
    <link rel="icon" href="/icon/internet">
    {generate_search_css()}
</head>
<body>
    {generate_modern_search_content(server, search_params, search_results, user, config)}
</body>
</html>"""
    
    return html

def parse_search_params(server):
    """Parse search parameters from the request."""
    params = {
        'text': '',
        'tags': False,
        'author': '',
        'tab': '',
        'sort': 'newest'
    }
    
    try:
        # Get query parameters
        if hasattr(server, 'parsed') and server.parsed:
            params['text'] = server.parsed.get('text', [''])[0]
            params['tags'] = 'tags' in server.parsed and server.parsed.get('tags', [''])[0] == 'on'
            params['author'] = server.parsed.get('author', [''])[0]
            params['tab'] = server.parsed.get('tab', [''])[0]
            params['sort'] = server.parsed.get('sort', ['newest'])[0]
    except:
        pass
    
    return params

def perform_search(params):
    """Perform the actual search based on parameters."""
    try:
        # Get all articles
        all_articles = Render.allArticles()
        results = []
        
        search_text = params['text'].lower()
        search_author = params['author'].lower()
        search_tab = params['tab'].lower()
        
        for url, article in all_articles.items():
            # Check if article matches search criteria
            matches = True
            
            # Text search
            if search_text:
                if params['tags']:
                    # Search in tags/hashtags
                    tags = article.get('tags', []) + article.get('hashtags', [])
                    tag_match = any(search_text in tag.lower() for tag in tags)
                    if not tag_match:
                        matches = False
                else:
                    # Search in title, description, and text
                    title = article.get('title', '').lower()
                    description = article.get('description', '').lower()
                    text = article.get('text', '').lower()
                    
                    if not (search_text in title or search_text in description or search_text in text):
                        matches = False
            
            # Author filter
            if search_author:
                author = (article.get('author', '') or article.get('by', '')).lower()
                if search_author not in author:
                    matches = False
            
            # Tab filter
            if search_tab:
                tab = article.get('tab', '').lower()
                if search_tab != tab:
                    matches = False
            
            if matches:
                results.append(article)
        
        # Sort results
        if params['sort'] == 'oldest':
            results.sort(key=lambda x: x.get('timestamp', 0))
        elif params['sort'] == 'popular':
            results.sort(key=lambda x: x.get('views', {}).get('amount', 0), reverse=True)
        else:  # newest (default)
            results.sort(key=lambda x: x.get('timestamp', 0), reverse=True)
        
        return results
        
    except Exception as e:
        print(f"Error performing search: {e}")
        return []

def generate_search_css():
    """Generate CSS for modern search pages."""
    return """
<style>
/* Reset body margins */
body {
    margin: 0 !important;
    padding: 0 !important;
}

/* Modern Search Page Styles */
.modern-search-page {
    min-height: 100vh !important;
    background: linear-gradient(135deg, #0f1419 0%, #1a1a2e 50%, #16213e 100%) !important;
    font-family: system-ui, -apple-system, sans-serif !important;
    color: #ffffff !important;
}

/* Header Section */
.modern-search-header {
    background: rgba(0, 0, 0, 0.3) !important;
    backdrop-filter: blur(10px) !important;
    border-bottom: 1px solid rgba(255, 255, 255, 0.1) !important;
    padding: 20px !important;
    position: sticky !important;
    top: 0 !important;
    z-index: 100 !important;
}

.modern-search-nav {
    max-width: 1400px !important;
    margin: 0 auto !important;
    display: flex !important;
    align-items: center !important;
    justify-content: space-between !important;
    gap: 20px !important;
}

.modern-back-button {
    background: rgba(255, 255, 255, 0.1) !important;
    backdrop-filter: blur(20px) !important;
    border: 1px solid rgba(255, 255, 255, 0.2) !important;
    border-radius: 8px !important;
    padding: 12px 20px !important;
    color: #ffffff !important;
    text-decoration: none !important;
    display: flex !important;
    align-items: center !important;
    gap: 8px !important;
    font-weight: 500 !important;
    transition: all 0.3s !important;
}

.modern-back-button:hover {
    background: rgba(255, 255, 255, 0.15) !important;
    transform: translateY(-2px) !important;
    box-shadow: 0 4px 12px rgba(0, 0, 0, 0.2) !important;
}

.modern-back-button img {
    width: 16px !important;
    height: 16px !important;
    filter: brightness(0) invert(1) !important;
}

/* Search Section */
.modern-search-section {
    padding: 48px 20px !important;
    max-width: 1200px !important;
    margin: 0 auto !important;
}

.modern-search-title {
    text-align: center !important;
    margin-bottom: 48px !important;
}

.modern-search-icon {
    width: 80px !important;
    height: 80px !important;
    margin: 0 auto 24px !important;
    background: rgba(255, 255, 255, 0.05) !important;
    backdrop-filter: blur(10px) !important;
    border-radius: 20px !important;
    padding: 16px !important;
    display: flex !important;
    align-items: center !important;
    justify-content: center !important;
}

.modern-search-icon img {
    width: 100% !important;
    height: 100% !important;
    filter: brightness(0) invert(1) !important;
}

.modern-page-title {
    font-size: 48px !important;
    font-weight: 700 !important;
    background: linear-gradient(135deg, #4c9eff 0%, #7c3aed 50%, #ec4899 100%) !important;
    -webkit-background-clip: text !important;
    -webkit-text-fill-color: transparent !important;
    background-clip: text !important;
    margin: 0 0 16px !important;
}

.modern-search-subtitle {
    font-size: 20px !important;
    color: #94a3b8 !important;
    margin: 0 !important;
}

/* Search Form */
.modern-search-form {
    background: rgba(255, 255, 255, 0.05) !important;
    backdrop-filter: blur(20px) !important;
    border: 1px solid rgba(255, 255, 255, 0.1) !important;
    border-radius: 16px !important;
    padding: 32px !important;
    margin-bottom: 48px !important;
}

.modern-form-row {
    display: flex !important;
    gap: 16px !important;
    margin-bottom: 24px !important;
    align-items: end !important;
}

.modern-form-group {
    flex: 1 !important;
}

.modern-form-label {
    display: block !important;
    font-size: 14px !important;
    font-weight: 600 !important;
    color: #e2e8f0 !important;
    margin-bottom: 8px !important;
}

.modern-form-input {
    width: 100% !important;
    background: rgba(255, 255, 255, 0.1) !important;
    border: 2px solid rgba(255, 255, 255, 0.2) !important;
    border-radius: 8px !important;
    padding: 16px 20px !important;
    font-size: 16px !important;
    color: #ffffff !important;
    outline: none !important;
    transition: all 0.3s !important;
    box-sizing: border-box !important;
}

.modern-form-input:focus {
    border-color: #4c9eff !important;
    box-shadow: 0 0 0 3px rgba(76, 158, 255, 0.2) !important;
    background: rgba(255, 255, 255, 0.15) !important;
}

.modern-form-input::placeholder {
    color: #64748b !important;
}

.modern-form-select {
    width: 100% !important;
    background: rgba(255, 255, 255, 0.1) !important;
    border: 2px solid rgba(255, 255, 255, 0.2) !important;
    border-radius: 8px !important;
    padding: 16px 20px !important;
    font-size: 16px !important;
    color: #ffffff !important;
    outline: none !important;
    transition: all 0.3s !important;
    box-sizing: border-box !important;
}

.modern-form-select:focus {
    border-color: #4c9eff !important;
    box-shadow: 0 0 0 3px rgba(76, 158, 255, 0.2) !important;
}

.modern-form-select option {
    background: #1f2937 !important;
    color: #ffffff !important;
}

.modern-checkbox-group {
    display: flex !important;
    align-items: center !important;
    gap: 12px !important;
    padding: 16px 0 !important;
}

.modern-checkbox {
    appearance: none !important;
    width: 20px !important;
    height: 20px !important;
    border: 2px solid rgba(255, 255, 255, 0.3) !important;
    border-radius: 4px !important;
    background: transparent !important;
    cursor: pointer !important;
    position: relative !important;
    transition: all 0.3s !important;
}

.modern-checkbox:checked {
    background: #4c9eff !important;
    border-color: #4c9eff !important;
}

.modern-checkbox:checked::after {
    content: '✓' !important;
    position: absolute !important;
    top: 50% !important;
    left: 50% !important;
    transform: translate(-50%, -50%) !important;
    color: #ffffff !important;
    font-size: 12px !important;
    font-weight: bold !important;
}

.modern-checkbox-label {
    font-size: 16px !important;
    color: #e2e8f0 !important;
    cursor: pointer !important;
}

.modern-search-button {
    background: linear-gradient(135deg, #4c9eff 0%, #7c3aed 100%) !important;
    border: none !important;
    border-radius: 8px !important;
    padding: 16px 32px !important;
    color: #ffffff !important;
    font-size: 16px !important;
    font-weight: 600 !important;
    cursor: pointer !important;
    display: flex !important;
    align-items: center !important;
    gap: 8px !important;
    transition: all 0.3s !important;
    text-decoration: none !important;
}

.modern-search-button:hover {
    transform: translateY(-2px) !important;
    box-shadow: 0 8px 24px rgba(76, 158, 255, 0.3) !important;
}

.modern-search-button img {
    width: 18px !important;
    height: 18px !important;
    filter: brightness(0) invert(1) !important;
}

/* Results Section */
.modern-results-section {
    margin-bottom: 48px !important;
}

.modern-results-header {
    display: flex !important;
    justify-content: space-between !important;
    align-items: center !important;
    margin-bottom: 32px !important;
}

.modern-results-title {
    font-size: 24px !important;
    font-weight: 600 !important;
    color: #ffffff !important;
    margin: 0 !important;
}

.modern-results-count {
    font-size: 16px !important;
    color: #94a3b8 !important;
}

.modern-current-search {
    background: rgba(76, 158, 255, 0.1) !important;
    border: 1px solid rgba(76, 158, 255, 0.3) !important;
    border-radius: 12px !important;
    padding: 20px !important;
    margin-bottom: 32px !important;
}

.modern-search-query {
    font-size: 18px !important;
    font-weight: 600 !important;
    color: #4c9eff !important;
    margin: 0 0 8px !important;
}

.modern-search-filters {
    display: flex !important;
    flex-wrap: wrap !important;
    gap: 12px !important;
}

.modern-filter-tag {
    background: rgba(76, 158, 255, 0.2) !important;
    color: #4c9eff !important;
    padding: 4px 12px !important;
    border-radius: 16px !important;
    font-size: 14px !important;
    font-weight: 500 !important;
}

/* Articles Grid (reuse from homepage) */
.modern-articles-grid {
    display: grid !important;
    grid-template-columns: repeat(auto-fill, minmax(380px, 1fr)) !important;
    gap: 24px !important;
}

/* Article Card Styles (same as homepage) */
.modern-article {
    background: rgba(255, 255, 255, 0.05) !important;
    backdrop-filter: blur(20px) !important;
    border: 1px solid rgba(255, 255, 255, 0.1) !important;
    border-radius: 12px !important;
    overflow: hidden !important;
    transition: all 0.3s !important;
    display: flex !important;
    flex-direction: column !important;
    position: relative !important;
}

.modern-article:hover {
    transform: translateY(-4px) !important;
    box-shadow: 0 20px 40px rgba(0, 0, 0, 0.3) !important;
    border-color: rgba(76, 158, 255, 0.5) !important;
}

.modern-article-badge {
    position: absolute !important;
    top: 16px !important;
    right: 16px !important;
    background: linear-gradient(135deg, #4c9eff 0%, #7c3aed 100%) !important;
    color: #ffffff !important;
    padding: 4px 12px !important;
    border-radius: 16px !important;
    font-size: 12px !important;
    font-weight: 600 !important;
    z-index: 10 !important;
}

.modern-article-thumbnail {
    width: 100% !important;
    height: 200px !important;
    object-fit: cover !important;
    background: linear-gradient(135deg, #1f2937 0%, #111827 100%) !important;
    display: block !important;
}

.modern-article-content {
    padding: 24px !important;
    flex: 1 !important;
    display: flex !important;
    flex-direction: column !important;
}

.modern-article-title {
    font-size: 20px !important;
    font-weight: 600 !important;
    color: #ffffff !important;
    margin: 0 0 16px !important;
    line-height: 1.4 !important;
    text-decoration: none !important;
    display: flex !important;
    align-items: center !important;
    gap: 8px !important;
}

.modern-article-title:hover {
    color: #4c9eff !important;
}

.modern-article-icon {
    width: 20px !important;
    height: 20px !important;
    filter: brightness(0) invert(1) !important;
    flex-shrink: 0 !important;
}

.modern-article-author-section {
    display: flex !important;
    align-items: center !important;
    gap: 12px !important;
    margin-bottom: 12px !important;
}

.modern-article-author-avatar {
    width: 40px !important;
    height: 40px !important;
    border-radius: 50% !important;
    object-fit: cover !important;
}

.modern-article-author {
    color: #e2e8f0 !important;
    text-decoration: none !important;
    font-size: 14px !important;
    font-weight: 500 !important;
}

.modern-article-author:hover {
    color: #4c9eff !important;
}

.modern-article-stats {
    display: flex !important;
    gap: 16px !important;
    margin-bottom: 16px !important;
    font-size: 14px !important;
    color: #94a3b8 !important;
}

.modern-stat {
    display: flex !important;
    align-items: center !important;
    gap: 4px !important;
}

.modern-article-preview {
    color: #cbd5e1 !important;
    line-height: 1.6 !important;
    font-size: 15px !important;
    flex: 1 !important;
    margin: 0 0 16px !important;
}

.modern-article-tags {
    display: flex !important;
    flex-wrap: wrap !important;
    gap: 8px !important;
    margin-top: auto !important;
}

.modern-tag {
    background: rgba(76, 158, 255, 0.1) !important;
    color: #4c9eff !important;
    padding: 4px 12px !important;
    border-radius: 16px !important;
    font-size: 12px !important;
    text-decoration: none !important;
    transition: all 0.2s !important;
}

.modern-tag:hover {
    background: rgba(76, 158, 255, 0.2) !important;
    transform: translateY(-1px) !important;
}

/* User Menu */
.modern-user-menu {
    position: fixed !important;
    top: 20px !important;
    right: 20px !important;
    z-index: 1000 !important;
}

.modern-user-button {
    background: rgba(255, 255, 255, 0.1) !important;
    backdrop-filter: blur(20px) !important;
    border: 1px solid rgba(255, 255, 255, 0.2) !important;
    border-radius: 12px !important;
    padding: 12px 20px !important;
    display: flex !important;
    align-items: center !important;
    gap: 12px !important;
    text-decoration: none !important;
    color: #ffffff !important;
    font-weight: 500 !important;
    transition: all 0.3s !important;
    position: relative !important;
}

.modern-user-button:hover {
    background: rgba(255, 255, 255, 0.15) !important;
    transform: translateY(-2px) !important;
    box-shadow: 0 8px 24px rgba(0, 0, 0, 0.3) !important;
}

.modern-user-avatar {
    width: 32px !important;
    height: 32px !important;
    border-radius: 50% !important;
    object-fit: cover !important;
}

.modern-user-icon {
    width: 20px !important;
    height: 20px !important;
    filter: brightness(0) invert(1) !important;
}

.modern-notification-badge {
    position: absolute !important;
    top: -4px !important;
    right: -4px !important;
    background: #ef4444 !important;
    color: #ffffff !important;
    font-size: 12px !important;
    font-weight: 700 !important;
    padding: 2px 6px !important;
    border-radius: 12px !important;
    min-width: 20px !important;
    text-align: center !important;
}

/* Empty State */
.modern-empty-state {
    text-align: center !important;
    padding: 80px 20px !important;
}

.modern-empty-icon {
    width: 80px !important;
    height: 80px !important;
    margin: 0 auto 24px !important;
    opacity: 0.3 !important;
    filter: brightness(0) invert(1) !important;
}

.modern-empty-title {
    font-size: 24px !important;
    font-weight: 600 !important;
    color: #e2e8f0 !important;
    margin: 0 0 12px !important;
}

.modern-empty-text {
    font-size: 16px !important;
    color: #94a3b8 !important;
}

/* Responsive */
@media (max-width: 768px) {
    .modern-search-nav {
        flex-direction: column !important;
        align-items: stretch !important;
        gap: 16px !important;
    }
    
    .modern-page-title {
        font-size: 36px !important;
    }
    
    .modern-form-row {
        flex-direction: column !important;
        align-items: stretch !important;
    }
    
    .modern-results-header {
        flex-direction: column !important;
        align-items: flex-start !important;
        gap: 16px !important;
    }
    
    .modern-articles-grid {
        grid-template-columns: 1fr !important;
    }
}
</style>
"""

def generate_modern_search_content(server, search_params, results, user, config):
    """Generate the modern search page content."""
    html = '<div class="modern-search-page">'
    
    # User menu
    html += generate_user_menu(user)
    
    # Header with navigation
    html += '<header class="modern-search-header">'
    html += '<div class="modern-search-nav">'
    
    # Back to home button
    html += f'''
    <a href="/" class="modern-back-button">
        <img src="/icon/left" alt="Back">
        <span>{safe_html(config.get("title", "Home"))}</span>
    </a>
    '''
    
    html += '</div>'
    html += '</header>'
    
    # Search section
    html += '<div class="modern-search-section">'
    
    # Title
    html += '''
    <div class="modern-search-title">
        <div class="modern-search-icon">
            <img src="/icon/search" alt="Search">
        </div>
        <h1 class="modern-page-title">Search</h1>
        <p class="modern-search-subtitle">Find articles, tags, and content</p>
    </div>
    '''
    
    # Search form
    html += generate_search_form(search_params)
    
    # Current search display (if there's an active search)
    if search_params['text'] or search_params['author'] or search_params['tab']:
        html += generate_current_search_display(search_params)
    
    # Results
    if search_params['text'] or search_params['author'] or search_params['tab']:
        html += generate_search_results(results, search_params)
    
    html += '</div>'  # End search section
    html += '</div>'  # End search page
    
    return html

def generate_search_form(params):
    """Generate the search form."""
    html = '''
    <form method="get" action="/search" class="modern-search-form">
        <div class="modern-form-row">
            <div class="modern-form-group">
                <label for="text" class="modern-form-label">Search Text</label>
                <input type="text" 
                       id="text" 
                       name="text" 
                       class="modern-form-input" 
                       placeholder="Enter search terms..." 
                       value="''' + safe_html(params['text']) + '''">
            </div>
            <div class="modern-form-group">
                <label for="sort" class="modern-form-label">Sort By</label>
                <select id="sort" name="sort" class="modern-form-select">
                    <option value="newest"''' + (' selected' if params['sort'] == 'newest' else '') + '''>Newest First</option>
                    <option value="oldest"''' + (' selected' if params['sort'] == 'oldest' else '') + '''>Oldest First</option>
                    <option value="popular"''' + (' selected' if params['sort'] == 'popular' else '') + '''>Most Popular</option>
                </select>
            </div>
            <button type="submit" class="modern-search-button">
                <img src="/icon/search" alt="Search">
                <span>Search</span>
            </button>
        </div>
        
        <div class="modern-form-row">
            <div class="modern-form-group">
                <label for="author" class="modern-form-label">Author</label>
                <input type="text" 
                       id="author" 
                       name="author" 
                       class="modern-form-input" 
                       placeholder="Filter by author..." 
                       value="''' + safe_html(params['author']) + '''">
            </div>
            <div class="modern-form-group">
                <label for="tab" class="modern-form-label">Category</label>
                <select id="tab" name="tab" class="modern-form-select">
                    <option value="">All Categories</option>
    '''
    
    # Add tab options
    try:
        tabs = Render.tabs()
        for tab_name, tab_info in tabs.items():
            selected = ' selected' if params['tab'] == tab_name else ''
            html += f'<option value="{safe_html(tab_name)}"{selected}>{safe_html(tab_info.get("title", tab_name))}</option>'
    except:
        pass
    
    html += '''
                </select>
            </div>
            <div class="modern-form-group">
                <div class="modern-checkbox-group">
                    <input type="checkbox" 
                           id="tags" 
                           name="tags" 
                           value="on" 
                           class="modern-checkbox"''' + (' checked' if params['tags'] else '') + '''>
                    <label for="tags" class="modern-checkbox-label">Search in tags only</label>
                </div>
            </div>
        </div>
    </form>
    '''
    
    return html

def generate_current_search_display(params):
    """Generate display of current search parameters."""
    html = '<div class="modern-current-search">'
    
    # Search query
    if params['text']:
        search_type = "tags" if params['tags'] else "content"
        html += f'<div class="modern-search-query">Searching {search_type}: "{safe_html(params["text"])}"</div>'
    
    # Filters
    filters = []
    if params['author']:
        filters.append(f'Author: {safe_html(params["author"])}')
    if params['tab']:
        try:
            tabs = Render.tabs()
            tab_title = tabs.get(params['tab'], {}).get('title', params['tab'])
            filters.append(f'Category: {safe_html(tab_title)}')
        except:
            filters.append(f'Category: {safe_html(params["tab"])}')
    if params['sort'] != 'newest':
        sort_labels = {'oldest': 'Oldest First', 'popular': 'Most Popular'}
        filters.append(f'Sort: {sort_labels.get(params["sort"], params["sort"])}')
    
    if filters:
        html += '<div class="modern-search-filters">'
        for filter_text in filters:
            html += f'<span class="modern-filter-tag">{filter_text}</span>'
        html += '</div>'
    
    html += '</div>'
    return html

def generate_search_results(results, params):
    """Generate the search results section."""
    html = '<div class="modern-results-section">'
    
    # Results header
    html += '<div class="modern-results-header">'
    html += '<h2 class="modern-results-title">Search Results</h2>'
    html += f'<div class="modern-results-count">{len(results)} result{"s" if len(results) != 1 else ""} found</div>'
    html += '</div>'
    
    if results:
        # Results grid
        html += '<div class="modern-articles-grid">'
        
        for article in results:
            # Prepare article data
            article_info = prepare_article_data(article)
            html += generate_article_card(article_info)
        
        html += '</div>'
    else:
        # Empty state
        html += '''
        <div class="modern-empty-state">
            <img src="/icon/search" alt="" class="modern-empty-icon">
            <h3 class="modern-empty-title">No results found</h3>
            <p class="modern-empty-text">Try adjusting your search terms or filters</p>
        </div>
        '''
    
    html += '</div>'
    return html

def prepare_article_data(article):
    """Prepare article data for display."""
    try:
        Accounts = Render.accounts()
        tabs = Render.tabs()
    except:
        Accounts = {}
        tabs = {}
    
    # Extract data
    title = article.get("title", "Untitled")
    url = article.get("url", "")
    author = article.get("author", "") or article.get("by", "")
    thumbnail = article.get("thumbnail", None)
    
    # Get icon from tab data
    tab = article.get("tab", "")
    icon = tabs.get(tab, {}).get("icon", "folder")
    
    # Get preview text
    preview = ""
    if "description" in article and article["description"]:
        preview = article["description"]
    elif "text" in article and article["text"]:
        preview = article["text"]
    
    # Try to load text from file if not in metadata
    if not preview and "url" in article:
        try:
            tab_article = article["url"].strip("/").split("/")
            if len(tab_article) == 2:
                text_path = Set.Folder() + f"/tabs/{tab_article[0]}/{tab_article[1]}/text.md"
                if os.path.exists(text_path):
                    with open(text_path, 'r') as f:
                        preview = f.read()
        except:
            pass
    
    # Limit preview length
    if preview and len(preview) > 200:
        preview = preview[:200] + "..."
    
    # Get stats
    views = article.get("views", {}).get("amount", 0)
    try:
        comments = len(article.get("comments", {}).get("comments", []))
    except:
        comments = 0
    
    # Get tags
    tags = article.get("tags", [])
    if not tags:
        tags = article.get("hashtags", [])
    
    # Check if unread (placeholder for now)
    is_unread = False
    
    # Get author info
    author_avatar = "/pictures/monkey.png"
    author_title = author
    if author in Accounts:
        author_avatar = Accounts[author].get("avatar", "/pictures/monkey.png")
        author_title = Accounts[author].get("title", author)
    
    return {
        'title': title,
        'url': url,
        'author': author,
        'author_title': author_title,
        'author_avatar': author_avatar,
        'preview': preview,
        'thumbnail': thumbnail,
        'icon': icon,
        'views': views,
        'comments': comments,
        'tags': tags,
        'is_unread': is_unread
    }

def generate_article_card(article):
    """Generate an article card HTML (same as homepage and tabs)."""
    html = f'<article class="modern-article">'
    
    # Unread badge
    if article.get('is_unread'):
        html += '<div class="modern-article-badge">Unread</div>'
    
    # Thumbnail
    if article.get('thumbnail'):
        html += f'<a href="{safe_html(article["url"])}"><img src="{safe_html(article["thumbnail"])}" alt="" class="modern-article-thumbnail"></a>'
    else:
        html += '<div class="modern-article-thumbnail"></div>'
    
    html += '<div class="modern-article-content">'
    
    # Title with icon
    icon = article.get('icon', 'scene')
    html += f'''<a href="{safe_html(article["url"])}" class="modern-article-title">
        <img src="/icon/{safe_html(icon)}" alt="" class="modern-article-icon">
        {safe_html(article["title"])}
    </a>'''
    
    # Author with avatar
    if article.get('author'):
        author_avatar = article.get('author_avatar', '/pictures/monkey.png')
        author_title = article.get('author_title', article['author'])
        html += f'''
        <div class="modern-article-author-section">
            <img src="{safe_html(author_avatar)}" alt="" class="modern-article-author-avatar">
            <a href="/account/{safe_html(article["author"])}" class="modern-article-author">{safe_html(author_title)}</a>
        </div>
        '''
    
    # Stats
    views = article.get('views', 0)
    comments = article.get('comments', 0)
    html += f'''
    <div class="modern-article-stats">
        <span class="modern-stat">👁 {views}</span>
        <span class="modern-stat">💬 {comments}</span>
    </div>
    '''
    
    # Preview
    if article.get('preview'):
        html += f'<p class="modern-article-preview">{safe_html(article["preview"])}</p>'
    
    # Tags
    if article.get('tags'):
        html += '<div class="modern-article-tags">'
        for tag in article['tags'][:5]:  # Limit to 5 tags
            html += f'<a href="/search?text=%23{safe_html(tag)}&tags=on" class="modern-tag">#{safe_html(tag)}</a>'
        html += '</div>'
    
    html += '</div>'
    html += '</article>'
    
    return html

def generate_user_menu(user):
    """Generate the user menu HTML."""
    html = '<div class="modern-user-menu">'
    
    if user:
        avatar = user.get("avatar", "/pictures/monkey.png")
        name = user.get("title", user.get("username", "Anonymous"))
        notifications = len(user.get("notifications", []))
        
        html += f'''
        <a href="/settings" class="modern-user-button">
            <img src="{safe_html(avatar)}" alt="Avatar" class="modern-user-avatar">
            <span>{safe_html(name)}</span>
        '''
        if notifications > 0:
            html += f'<span class="modern-notification-badge">{notifications}</span>'
        html += '</a>'
    else:
        html += '''
        <a href="/login" class="modern-user-button">
            <img src="/icon/unlock" alt="Login" class="modern-user-icon">
            <span>Login</span>
        </a>
        '''
    
    html += '</div>'
    return html

def safe_html(text):
    """Escape HTML to prevent XSS attacks."""
    if not text:
        return ''
    return html.escape(str(text))
